# -*- coding: utf-8 -*-
import scrapy


class DocumentItem(scrapy.Item):
    title = scrapy.Field()
    file = scrapy.Field()
    release = scrapy.Field()
    category = scrapy.Field()
    date = scrapy.Field()


class SoftwareItem(scrapy.Item):
    file_urls = scrapy.Field()
    files = scrapy.Field()



# -*- coding: utf-8 -*-
import os


#  User configuration & settings
XCHANGE_USERNAME = os.getenv('XCHANGE_USER', 'username@example.com')
XCHANGE_PASSWORD = os.getenv('XCHANGE_PASS', 'password')
LOG_ENABLED = True
LOG_FILE = None
LOG_FORMAT = '%(message)s'  # '%(asctime)s [%(name)s] %(levelname)s: %(message)s'
LOG_LEVEL = 'DEBUG'
LOG_STDOUT = True
DEPTH_STATS = False
DEPTH_STATS_VERBOSE = False
DOWNLOADER_STATS = True
MEMDEBUG_ENABLED = False
MEMUSAGE_ENABLED = False
MEMUSAGE_LIMIT_MB = 0
AUTOTHROTTLE_DEBUG = False
TELNETCONSOLE_ENABLED = False
DOWNLOAD_TIMEOUT = 1800
DOWNLOAD_MAXSIZE = 4294967296  # 4GB
DOWNLOAD_WARNSIZE = 1073741824  # 1GB
DOWNLOAD_DRYRUN = False
FILES_STORE = os.getenv('XCHANGE_STORE', os.path.expanduser('~'))
REQUEST_METHOD = 'GET'
RETRY_ENABLED = True
RETRY_TIMES = 10
RETRY_HTTP_CODES = [500, 502, 503, 504, 400, 408]



#  Optimized defaults for best performance & kindness to xchange.
BOT_NAME = 'scrapy-broadsoftxchange'
USER_AGENT = 'scrapy-broadsoftxchange (+https://github.com/lukebeer/scrapy-broadsoftxchange)'
SPIDER_MODULES = ['broadsoftxchange.spiders']
ITEM_PIPELINES = {'broadsoftxchange.pipelines.SoftwarePipeline': 1}
RANDOMIZE_DOWNLOAD_DELAY = False
REACTOR_THREADPOOL_MAXSIZE = 8
REDIRECT_MAX_TIMES = 20
CONCURRENT_REQUESTS = 8
CONCURRENT_ITEMS = 50
DEPTH_LIMIT = 0
DEPTH_PRIORITY = 0
DOWNLOAD_DELAY = 0
CONCURRENT_REQUESTS_PER_DOMAIN = 8
CONCURRENT_REQUESTS_PER_IP = 8
COOKIES_ENABLED = True
AUTOTHROTTLE_ENABLED = True
AUTOTHROTTLE_START_DELAY = 5
AUTOTHROTTLE_MAX_DELAY = 60



# -*- coding: utf-8 -*-
import scrapy
from scrapy import spiders, log

from broadsoftxchange.items import SoftwareItem
from broadsoftxchange import settings


class SoftwareSpider(spiders.CrawlSpider):
    name = "software"
    http_user = settings.XCHANGE_USERNAME
    http_pass = settings.XCHANGE_PASSWORD
    start_urls = [
        'http://xchange.broadsoft.com/XchangeRepos/GA',
    ]

    def parse(self, response):
        self.log(response.url, level=scrapy.log.INFO)
        if response.headers['Content-Type'] != 'text/plain':
            if not settings.DOWNLOAD_DRYRUN:
                yield SoftwareItem(file_urls=[response.url])
        else:
            if 'Contents of this Folder:' in response.body:
                for link in response.body.split('\n')[1:-1]:
                    next = '%s/%s' % (response.url, link.strip())
                    yield scrapy.Request(url=next, method=settings.REQUEST_METHOD, callback=self.parse)
            else:
                self.log(response.url, level=scrapy.log.INFO)
                if not settings.DOWNLOAD_DRYRUN:
                    yield SoftwareItem(file_urls=[response.url])



# -*- coding: utf-8 -*-
import scrapy
from scrapy import spiders, linkextractors

from broadsoftxchange.items import DocumentItem


class Documentation(spiders.CrawlSpider):
    name = 'documentation'
    allowed_domains = ['xchange.broadsoft.com']
    login_page = 'http://xchange.broadsoft.com/php/xchange/'
    start_urls = ['http://xchange.broadsoft.com/php/xchange/support/broadworks/documentation?page=1']

    rules = (
        spiders.Rule(
            linkextractors.LinkExtractor(allow=r'/php/xchange/support/broadworks/documentation\?page=[0-9]+',
                                         unique=False, canonicalize=False),
            callback='parse_item',
            follow=True,
        ),
    )

    def start_requests(self):
        yield scrapy.Request(
            url=self.login_page,
            callback=self.login,
            dont_filter=False,
        )

    def login(self, response):
        return scrapy.FormRequest.from_response(
            response,
            formxpath='//*[@id="user-login-form"]',
            formdata={
                'name': self.settings['XCHANGE_USERNAME'],
                'pass': self.settings['XCHANGE_PASSWORD']
            },
            callback=self.check_login_response,
        )

    def check_login_response(self, response):
        try:
            if 'Configure your rss feeds' in response.body:
                self.log("Successfully logged in")
                for url in self.start_urls:
                    yield scrapy.Request(url=url, dont_filter=True)
        except:
            self.log("Login failed")
            return

    def parse_item(self, response):
        items = []
        links = response.xpath("//tr[@class='odd rowclick even']|//tr[@class='even rowclick odd']")
        for row in links:
            self.log(row)
            item = DocumentItem()
            item['title'] = row.select("td[3]/a/text()").extract()
            item['file'] = row.select("td[2]/a/@href").extract()
            item['release'] = row.select("td[6]/text()").extract()
            item['category'] = row.select("td[5]/text()").extract()
            item['date'] = row.select('td[7]/text()').extract()
            self.log(item)
            items.append(item)
        return items



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(
    name='scrapy-broadsoftxchange',
    version="1.4",
    description="Download documents and published software from Broadsoft Xchange",
    long_description="",
    keywords='scrapy broadsoft broadworks xchange',
    url='https://github.com/lukebeer/scrapy-broadsoftxchange',
    author='Luke Berezynskyj (Aka Beer)',
    author_email='mail@luke.beer',
    license='MIT',
    include_package_data=True,
    packages=find_packages(),
    install_requires=[
        'distribute',
        'scrapy',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Telecommunications Industry',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
    ]
)



# -*- coding: utf-8 -*-
try:
    from cStringIO import StringIO as BytesIO
except ImportError:
    from io import BytesIO
from urlparse import urlparse

from scrapy.pipelines.files import FilesPipeline

try:
    from cStringIO import StringIO as BytesIO
except ImportError:
    from io import BytesIO

from scrapy.http import Request


class SoftwarePipeline(FilesPipeline):
    def file_path(self, request, response=None, info=None):
        if not isinstance(request, Request):
            url = request
        else:
            url = request.url
        bits = urlparse(url)
        return bits.netloc + bits.path
